#!/bin/bash

#######################################################
#
# NAME: Tilde Art Exchange
# VERSION: 1
# AUTHOR: malvarma
# MASTODON: @malvarma@tiny.tilde.website
#
# DESCRIPTION: 
#   New to programming. This is my first game.
#   If you have suggestions on how I can improve the
#   code, please msg me. Feedback is appreciated!
#
#######################################################


#====================Settings====================#

# Location of game data
csv_location=data/tax.csv
backup_location=data/.tax.back
readme_location=data/readme.txt

# Currency Settings
currencies=("poems" "jokes" "compliments")
max_price=5

# Select Display Settings
PS3='Make your selection:'
COLUMNS=12

#====================Functions====================#

view_offerings(){
    echo ""
    while IFS="," read -r rec_id rec_name rec_owner rec_price rec_curr
    do
      echo "Item $rec_id - $rec_name"
      echo "Owner: $rec_owner"
      echo "Price: $rec_price $rec_curr"
      echo ""
    done < ${csv_location}
}

write_cell () {

    # Verifies ownership of item, overwrites cell in the backup version, and syncs to live version
    # USAGE: write_cell [ROW] [COLUMN] [NEWVALUE]

    username=$(whoami)
    owner=$(awk -v LOOKUPVAL=$1 'BEGIN{FS=OFS=","} $1 == LOOKUPVAL { print $3 }' < ${csv_location})
    if ! [ $username == $owner ] ; then
        echo "Permission denied: You are not the owner of this item."
        break
    fi

    awk -v r=$1 -v c=$2 -v val=$3 'BEGIN{FS=OFS=","} NR==r{$c=val} 1' ${csv_location} > ${backup_location}

    cp -p ${backup_location} ${csv_location} &>/dev/null # Silence error re: preserving times. I just need to preserve permissions.
}

set_price(){

    view_offerings
    echo "Edit price of which item? Enter number"    
    read item_to_price

    echo "Price must be between 1 and ${max_price}. Enter any other value to list as not for sale."
    echo "Enter new price" 
    read new_price
    if [ $new_price -lt 1 ] || [ $new_price -gt $max_price ] || ! [[ $new_price =~ ^[0-9]+$ ]]; then
        new_price='not'
        new_currency='selling'
    else
        echo "Select a currency"
        select new_currency in "${currencies[@]}"; do


            if [[ " ${currencies[@]} " =~ " ${new_currency} " ]]; then
                break
            fi
            echo "It's rude to order off the menu. Pick a valid option."
        done
    fi

    write_cell ${item_to_price} 4 ${new_price}
    write_cell ${item_to_price} 5 ${new_currency}

    echo "We have updated our records to reflect the new price. Thank you, and good bye." 
}

transfer_item(){

    view_offerings
        
    echo "Transfer which item? Enter number"    
    read item_to_transfer

    echo "Transfer to which user? Type their username exactly as it appears in /etc/passwd"    
    read new_user

    # Check if user exists
    if ! id "$new_user" &>/dev/null; then
        echo 'User not found. You must type their username exactly as it appears in /etc/passwd - Please check for typos and try again. '
        break
    fi

    # Confirm transaction
    echo "Are you sure you want to transfer this item to $new_user ? This transaction can not be undone. Please type y to confirm, or any other character to cancel."
        read transfer_confirmation

    if ! [ $transfer_confirmation == 'y' ]; then
        echo "Transfer cancelled."
        break
    fi

    write_cell ${item_to_transfer} 4 "not"
    write_cell ${item_to_transfer} 5 "selling"
    write_cell ${item_to_transfer} 3 ${new_user}


    echo "Item transfer complete. Thank you, and good bye." 
}

#====================Main====================#

# Create backup on first run of the script

if ! test -f ${backup_location}; then
    cp -p ${csv_location} ${backup_location}
fi

# If CSV has been edited outside the script, restore backup.

if ! cmp -s ${csv_location} ${backup_location}; then
    cp -p ${backup_location} ${csv_location}
fi

# Main Menu

echo "Welcome to Tilde Art Exchange"

menu=("About Us" "View Offerings" "Set Item Price" "Transfer Item" "Quit")

select menu_selection in "${menu[@]}"; do
    case $menu_selection in
        "About Us")
            nano -v --softwrap ${readme_location}
            ;;
        "View Offerings")
            view_offerings
            echo "To buy an item, please contact the item's owner."
            echo "Thank you for browsing our collection. Good bye."
	        break
            ;;
        "Set Item Price")
            set_price
	        break
            ;;
        "Transfer Item")
            transfer_item
	        break
            ;;
	    "Quit")
	        echo "Good bye."
	        exit
	        ;;
            *) echo "invalid option $REPLY";;
    esac
done
